#!/usr/bin/perl
use warnings;
use strict;
use File::Basename;

##############################################################################
#                                  WARNING                                   #
# *keyword: External="[job_name]_TSSMM" is always required in the route line #
#           of the corresponding GAUSSIAN input file. [job_name] should be   #
#           replaced by the GAUSSIAN input file name.                        #
#                                                                            #
# *keyword: opt=(NoMicro) is required for geometry optimization.             #
#                                                                            #
# *keyword: Freq cannot be used for starting a Frequency calculation         #
#           automatically by the end of a geometry optimization. A seperate  #
#           frequency calculation with a new GAUSSIAN input file is          #
#           necessary for the freqency analysis of converged geometry.       #
##############################################################################


#---------------------------------------------------------------------------------------------------

#
# path and directory setup
#
our $GAUSSIAN    = '/panfs/roc/msisoft/gaussian/g09.e01_mesabi/g09/g09'; # 'which g09'
our $TSSMM       = '/panfs/roc/groups/8/truhlard/yang3227/TSSMM';
our $JOB         = '/panfs/roc/groups/8/truhlard/yang3227/TSSMM_JOB';
our $SCRATCH     = '/scratch.global/yang3227/TSSMM_SCRDIR'; # preferably user defined scratch directory for TSSMM
#
# calculation setup
#
our $nproc       = '%nproc=12';
our $mem         = '%mem=27000mb';
our $functional  = 'uM06L';
our $option      = 'Int=UltraFine NoSymm scf=(xqc,maxconventionalcyc=100) guess=mix';
our $const       = #; #spin orbit coupling constant in Hartree
our @multi       = (1,3); #spin multiplicity of the (1st,2nd) state of choice

#---------------------------------------------------------------------------------------------------

our $main_input;
our $script;

{
	opendir my $dir, "$JOB" or die "Can't open directory: $!";
	my @files = readdir $dir;
	closedir $dir;
	my $i = 0;
	foreach my $file (@files) {
		if (($file =~ m/\.gjf$/i) or ($file =~ m/\.com$/i)) {
			$i = $i+1;
			$main_input = $file;
			open (FILE0, "$main_input") or die "Can't open $main_input: $!";
			my @line = <FILE0>;
			close FILE0;
			$main_input =~ s/(.*)\.(.*)/$1/;
			if (grep(/\b$main_input\_TSSMM\b/,@line) == 0 ) {
				print "Gaussian input $file missing keyword: external=\"$main_input\_TSSMM\"\n";
				exit;
			}
		}
		elsif ($file =~ m/\.pbs$/i) {
			$script = $file;
		}
	}
	if ($i == 0) {
		print "missing Gaussian input file\n";
		exit;
	}
	undef $i;
}

{
	system ("cp $TSSMM/TSSMM_sub $JOB/TSSMM_sub");
	open (my $in, '<', "$JOB/TSSMM_sub") or die "Can't read TSSMM_sub: $!";
	open (my $out, '>', "$JOB/$main_input\_TSSMM") or die "Can't write $main_input\_TSSMM: $!";
	while (<$in>) {
		print $out $_;
		if ($_ =~ /^# SETUP/) {
			print $out "#\nour \$GAUSSIAN = '$GAUSSIAN';\nour \$JOB = '$JOB';\nour \$SCRATCH = '$SCRATCH';\n";
			print $out "our \$nproc = '$nproc';\nour \$mem = '$mem';\n";
			print $out "our \$functional = '$functional';\nour \$option = '$option';\n";
			print $out "our \$const = $const;\n\$multi[0] = $multi[0];\n\$multi[1] = $multi[1];\n";
		}
	}
	unlink "$JOB/TSSMM_sub";
	close $out;
	system ("chmod u+x $main_input\_TSSMM");
}

{	
	if ($script) {
		my $JOBbase = basename($JOB);
		my ($label1,$label2);
		open (FILE1, "$script");
		my @line = <FILE1>;
		close FILE1;
		if (grep(/$JOBbase:\$PATH/,@line) eq 0 ) {
			$label1 = 1;
		}
		if (grep(/export GAUSS_SCRDIR/i,@line) eq 0 ) {
			$label2 = 1;
		}
		if (($label1) or ($label2)) {
			system ("cp $script Gs_temp");
			open (my $in, '<', "Gs_temp");
			open (my $out, '>', "$script");
			while (<$in>) {
				print $out "export PATH=$JOB:\$PATH\n" if ((/\bg09\b/i) and (/$main_input/) and ($label1));
				print $out "mkdir -p $SCRATCH\n" if ((/\bg09\b/i) and (/$main_input/) and ($label2));
				print $out "export GAUSS_SCRDIR=$SCRATCH\n" if ((/\bg09\b/i) and (/$main_input/) and ($label2));
				print $out $_;
			}
		unlink "Gs_temp";
		close $out;
		}
	}	
}



